#include <windows.h>
#include "OptimusMiniHelper.h"

extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	return 1;
}

typedef void (__stdcall *TEventHandler)( int );

class CDllEventHandler : public CEventHandler
{
public:

	TEventHandler onKeyDown;
	TEventHandler onDeviceStateChanged;

	CDllEventHandler()
	{
		onKeyDown = onDeviceStateChanged = NULL;
	}

	virtual void OnKeyDown( int p_nKey )
	{
		if (onKeyDown != NULL)
		{
			onKeyDown( p_nKey );
		}
	}

	virtual void OnDeviceStateChanged( BOOL p_bState )
	{
		if (onDeviceStateChanged != NULL)
		{
			onDeviceStateChanged( p_bState ? 1:0 );
		}
	}
} EventHandler;

#define DLL_EXPORT extern "C" __declspec(dllexport) 

COptimusMiniHelper* oOptimus = NULL;

DLL_EXPORT void Init()
{
	if (oOptimus == NULL)
	{
		oOptimus = new COptimusMiniHelper(); 
		oOptimus->RegisterEventHandler( &EventHandler );
		oOptimus->Init();
	}
}
DLL_EXPORT void Close()
{
	if (oOptimus != NULL)
	{
		oOptimus->Close();
		delete oOptimus;
		oOptimus = NULL;
	}
}

DLL_EXPORT void RegisterEventHandler( TEventHandler ok, TEventHandler od )
{
	EventHandler.onKeyDown = ok;
	EventHandler.onDeviceStateChanged = od;		
}

DLL_EXPORT int IsAlive()
{
	return oOptimus != NULL && oOptimus->m_bDeviceAlive ? 1 : 0;
}

DLL_EXPORT int SendCommand( BYTE c1, BYTE c2 )
{
	return oOptimus != NULL ? oOptimus->SendCommand( c1, c2 ) : COMMAND_FAILED;
}

DLL_EXPORT int SendCommandBlocking( BYTE c1, BYTE c2 )
{
	return oOptimus != NULL ? oOptimus->SendCommand( c1, c2, TRUE ) : COMMAND_FAILED;
}

DLL_EXPORT int ShowPicture( int p_nButton, LPBYTE p_lpPictureBytes )
{
	return oOptimus != NULL ? oOptimus->ShowPicture( p_nButton, p_lpPictureBytes ) : COMMAND_FAILED;
}

DLL_EXPORT int ShowPictureBlocking( int p_nButton, LPBYTE p_lpPictureBytes )
{
	return oOptimus != NULL ? oOptimus->ShowPicture( p_nButton, p_lpPictureBytes, TRUE ) : COMMAND_FAILED;
}

#ifdef FEATURE_GAMMA
DLL_EXPORT void DeviceGamma( double p_dGamma )
{
	if (oOptimus != NULL)
	{
		oOptimus->DeviceGamma( p_dGamma );
	}
}
#endif

#ifdef FEATURE_COLOR_TEMPERATURE
DLL_EXPORT void DeviceColorTemperature( int p_iTemperature )
{
	if (oOptimus != NULL)
	{
		oOptimus->DeviceColorTemperature( p_iTemperature );
	}
}
#endif

#ifdef FEATURE_LAYOUT
DLL_EXPORT void ChangeLayout()
{
	if (oOptimus != NULL)
	{
		oOptimus->ChangeLayout();
	}
}

#endif

#ifdef FEATURE_REPAINT
DLL_EXPORT void RepaintPictures()
{
	if (oOptimus != NULL)
	{
		oOptimus->RepaintPictures();
	}
}
#endif

