#include "StdAfx.h"
#include "conio.h"
#include "BmpTool.h"

// Helper uses separate thread for device communication
#define HELPER

#ifdef HELPER

#include "OptimusMiniHelper.h"
COptimusMiniHelper* lpOptimus = NULL;

#else

#include "OptimusMini.h"
COptimusMini* lpOptimus = NULL;

#endif

CBmpTool objBmp[3];

DWORD dwLastKeyDown;
DWORD dwLastKeyDownTime;

#ifdef HELPER
class CHandler : public CEventHandler
{
public:
	void OnKeyDown( int p_nKey )
	{
		printf("keydown %d\n", p_nKey);
		
		// Multiple presses filter
		if( ::GetTickCount() - dwLastKeyDownTime > 300 || dwLastKeyDown != p_nKey )
		{
			dwLastKeyDownTime = ::GetTickCount();
			dwLastKeyDown = p_nKey;

			lpOptimus->ShowPicture( p_nKey, objBmp[p_nKey-1].m_pData + SCREEN_BUFFER_SIZE * (rand() % 3) );
		}
	}

	void OnDeviceStateChanged( BOOL p_bState )
	{
		printf( "CHandler::OnDeviceStateChanged %s\n", p_bState ? "alive" : "dead" );
		if (p_bState){
			// Turn Optimus ON
			printf( "on\n" );
			lpOptimus->SendCommand( 2,0 );

			// Setting max brightness - 60%
			printf( "brightness set\n" );
			lpOptimus->SendCommand( 9, 60 );
		}
	}
};

#endif // HELPER

void SetBrightness(int value)
{
	lpOptimus->SendCommand( 9, value ); 
	printf( "brightness %d\n", value );
}

void SetGamma(double value)
{
#ifdef FEATURE_GAMMA
	lpOptimus->DeviceGamma(value);
#ifdef HELPER
	lpOptimus->RepaintPictures();
#endif
	printf( "gamma %.2f\n", value );
#endif
}

void SetColorTemperature(int value)
{
#ifdef FEATURE_COLOR_TEMPERATURE
	lpOptimus->DeviceColorTemperature(value);
#ifdef HELPER
	lpOptimus->RepaintPictures();
#endif
	printf( "Color temperature %d\n", value );
#endif
}

DWORD WINAPI OptimusLife( LPVOID lpParam )
{
	// Picture displaying
	lpOptimus->ShowPicture( 1, objBmp[0].m_pData + SCREEN_BUFFER_SIZE );
	lpOptimus->ShowPicture( 2, objBmp[1].m_pData + SCREEN_BUFFER_SIZE );
	lpOptimus->ShowPicture( 3, objBmp[2].m_pData + SCREEN_BUFFER_SIZE );

	// Displaying device ID using blocking SendCommand call
	printf("Device ID: %d\n", lpOptimus->SendCommand(8,0, TRUE ) );

	BOOL bTurnedOn = TRUE;

	double g=0.65;
	int    t=8500;

	for(;;)
	{
		int nKey = getch();

		switch( nKey )
		{
			case '1': SetBrightness( 20 ); break;
			case '2': SetBrightness( 40 ); break;
			case '3': SetBrightness( 60 ); break;

			case '4': g+=0.05; SetGamma( g ); break;
			case '5': g-=0.05; SetGamma( g ); break;
			case '6': g=0.65;  SetGamma( g ); break;

			case '7': t+=500; SetColorTemperature( t ); break;
			case '8': t-=500; SetColorTemperature( t ); break;
			case '9': t=8500; SetColorTemperature( t ); break;

			case 'q': 
				lpOptimus->Init();
				break;

			case 'z':
				lpOptimus->ShowPicture( 1, objBmp[rand()%3].m_pData );
				break;

			case 27:
				// esc - turn OFF/ON
				printf( "Turn %s\n", bTurnedOn ? "off" : "on" );
				lpOptimus->SendCommand( bTurnedOn ? 3 : 2, 0 );
				bTurnedOn = !bTurnedOn;

				if( bTurnedOn )
				{
					// If turned ON, sending image from device RAM to OLED.
					lpOptimus->SendCommand( 4, 1 );
					lpOptimus->SendCommand( 4, 2 );
					lpOptimus->SendCommand( 4, 3 );
				}
				break;

			case 3:
				printf( "ThreadProc returned\n" );
				exit(0);

			default:
				printf( "press ctrl+c to exit\n" );
		}
	}


	return 0;
}


int main(int argc, char* argv[])
{
	char *bmpName[3] = { 
		"..\\img\\key_01.bmp",
		"..\\img\\key_02.bmp",
		"..\\img\\key_03.bmp",
	};

	for (int i=0;i<3;i++){
		objBmp[i].Load(bmpName[i]);
	}

#ifdef HELPER
	COptimusMiniHelper dev;
	CHandler handler;
	dev.RegisterEventHandler( &handler );
	dev.Init();
#else
	COptimusMini dev;
	dev.Init();
	dev.SendCommand( 2,0 );
#endif

	lpOptimus = &dev;

	// waiting until device is inited
	while( !dev.m_bDeviceAlive )
	{
		Sleep(10);
	}

	// Starting separate thread
	DWORD dwThreadId;
	::CreateThread( NULL, 0, OptimusLife, NULL, 0, &dwThreadId );

	// Main thread just rotating progress
	char progress[] = "-\\|/";

	for( ;; )
	{
		for( char* pc=progress; *pc != 0; pc++ )
		{
			printf( "%c\b", *pc );
			::Sleep( 100 );
		}
	}

	return 0;
}
