#pragma once

// **************** Device IO settings ****************

#define PORT_TIMEOUT 4000		// timeout for all write operations and default timeout for read operations during command execution
#define PORT_RETRY 3			// number of times we attepmt to resent the command before failing

// Overlapped support does not work under Windows 98. 
// Overlapped mode is used in OptimusMiniHelper device thread to wait either for new command or device keydown

#define PORT_OVERLAPPED			// overlapped support
#define PORT_CAN_USE_OVERLAPPED ( ::GetVersion() < 0x80000000 )

// **************** Class for input/output operations ****************

class CPort
{

private:
	char m_szPortName[260];
	DWORD m_dwTimeout;

#ifdef PORT_OVERLAPPED
	OVERLAPPED m_objOverlapped;	// overlapped mode usage
#endif

public:
	HANDLE m_hHandle;

public:
	CPort() 
	{
		*m_szPortName = '\0';
		m_hHandle = INVALID_HANDLE_VALUE;
	}

	~CPort()
	{
		Close();
	}

	BOOL Open( LPSTR p_szPortName );
	void Close();

	void SetTimeout( DWORD p_nTimeout );

	BOOL WriteFile( LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten, LPOVERLAPPED lpOverlapped = NULL );
	BOOL ReadFile( LPVOID lpBuffer, DWORD nNumberOfBytesToRead, LPDWORD lpNumberOfBytesRead, LPOVERLAPPED lpOverlapped = NULL );

	BOOL SetDCB();

	inline BOOL IsOpened(){ return m_hHandle != INVALID_HANDLE_VALUE; };
	inline char *GetPortName(){ return IsOpened() ? m_szPortName : NULL; };
};
